@@ -379,15 +379,12 @@
            ;; It is OK to compare type variables with eqv? here.
            (if (or (and type-check? (not (type-dispatched? to)))
                    (not (for-all eqv? subreprs old-member-types)))
-             ;; We must not overwrite the singleton object in procedure
-             ;; do-bind-type-vars00 with a class. Otherwise we would have
-             ;; to different (by eq?) objects representing the same class.
              (if type-check?
                (let ((to-type (get-union-of-types binder subreprs)))
                  (if (is-tt-union? to-type)
-                   to-type
-                   (make-union-expression0 (list to-type))))
-               (make-union-expression0 subreprs))
+                   (check-none-union to-type '())
+                   to-type))
+               (check-none-union (make-union-expression0 subreprs) '()))
              to)))
       result)))
   
@@ -807,3 +804,29 @@
 
 
 (set! contains-type-modifiers-fwd? contains-type-modifiers?)
+
+
+(define (check-none-union tt l-visited)
+  (cond
+    ((memq tt l-visited) tt)
+    ((and (is-tt-union? tt) (not (is-incomplete-object0? tt)))
+     (let* ((l-member-types (tno-field-ref tt 'l-member-types))
+            (l-new-visited (cons tt l-visited))
+            (l-member-types2
+             (map
+              (lambda (tt2) (check-none-union tt2 l-new-visited))
+              l-member-types)))
+       (if (memq tt-none l-member-types2) tt-none tt)))
+    (else tt)))
+
+       
+(define (nonoverwritten-type? to)
+  (assert (is-target-object? to))
+  (or (eq? to tt-none)
+      (eq? (get-entity-type to) tc-class)
+      (is-tt-unit-type? to)))
+
+
+(set! nonoverwritten-type-fwd? nonoverwritten-type?)
+
+
